@@ -1,32 +1,37 @@
+import importlib.metadata as importlib_metadata
 import json
-import subprocess
-import yaml
 import os
-import pkg_resources
 import signal
-from flask import jsonify, request, abort, make_response, Blueprint
-from werkzeug.exceptions import InternalServerError, HTTPException
+import subprocess
 from typing import Any, Dict
 
 import structlog
+import yaml
+from flask import Blueprint, abort, jsonify, make_response, request
+from werkzeug.exceptions import HTTPException, InternalServerError
 
+from insightconnect_plugin_runtime.api.schemas import (
+    ActionTriggerDetailsSchema,
+    ConnectionDetailsSchema,
+    PluginInfoSchema,
+    TaskDetailsSchema,
+)
 from insightconnect_plugin_runtime.exceptions import (
     ClientException,
-    ServerException,
-    LoggedException,
     ConnectionTestException,
+    LoggedException,
     PluginException,
-)
-from insightconnect_plugin_runtime.api.schemas import (
-    PluginInfoSchema,
-    ActionTriggerDetailsSchema,
-    TaskDetailsSchema,
-    ConnectionDetailsSchema,
+    ServerException,
 )
 from insightconnect_plugin_runtime.util import OutputMasker
+from uuid import UUID
 
 logger = structlog.get_logger("plugin")
 ORG_ID = "X-IPIMS-ORGID"
+INT_ID = "X-INTEGRATION-ID"
+
+PLUGIN_SPEC_DOCKERFILE = "/python/src/plugin.spec.yaml"
+PLUGIN_SPEC_PACK = "/workspace/plugin.spec.yaml"
 
 
 # Must be defined at application level in order to capture all 404's with blueprints,
@@ -170,17 +175,22 @@ class Endpoints:
                 500:
                   description: Unexpected error
             """
+
             self.logger.info("Plugin task beginning execution...")
             input_message = request.get_json(force=True)
             self.logger.debug("Request input: %s", input_message)
             Endpoints.validate_action_trigger_task_empty_input(input_message)
             Endpoints.validate_action_trigger_task_name(input_message, name, "task")
+
             # No validation on the plugin custom config to leave this as configurable as possible.
             # `add_plugin_custom_config` will pass any available values to the plugin for interpretation.
             input_message = self.add_plugin_custom_config(
-                input_message, request.headers.get(ORG_ID)
+                input_message,
+                request.headers.get(ORG_ID, ""),
+                request.headers.get(INT_ID, ""),
+                name,
             )
-            output = self.run_action_trigger_task(input_message)
+            output = self.run_action_trigger_task(input_message, mask_output=False)
             self.logger.info("Plugin task finished execution...")
             return output
 
@@ -329,9 +339,7 @@ class Endpoints:
                   description: InsightConnect Plugin Information to be returned
                   schema: PluginInfoSchema
             """
-            plugin_spec_json = Endpoints.load_file_json_format(
-                "/python/src/plugin.spec.yaml"
-            )
+            plugin_spec_json = Endpoints.load_plugin_spec_file()
             plugin_info_fields = [
                 "name",
                 "description",
@@ -430,9 +438,7 @@ class Endpoints:
                 400:
                   description: Bad request
             """
-            plugin_spec_json = Endpoints.load_file_json_format(
-                "/python/src/plugin.spec.yaml"
-            )
+            plugin_spec_json = Endpoints.load_plugin_spec_file()
             Endpoints.action_trigger_task_exists(plugin_spec_json, "actions", name)
             return jsonify(
                 ActionTriggerDetailsSchema().dump(
@@ -459,9 +465,7 @@ class Endpoints:
                 400:
                   description: Bad request
             """
-            plugin_spec_json = Endpoints.load_file_json_format(
-                "/python/src/plugin.spec.yaml"
-            )
+            plugin_spec_json = Endpoints.load_plugin_spec_file()
             Endpoints.action_trigger_task_exists(plugin_spec_json, "tasks", name)
             return jsonify(
                 TaskDetailsSchema().dump(plugin_spec_json.get("tasks").get(name))
@@ -486,9 +490,7 @@ class Endpoints:
                 400:
                   description: Bad request
             """
-            plugin_spec_json = Endpoints.load_file_json_format(
-                "/python/src/plugin.spec.yaml"
-            )
+            plugin_spec_json = Endpoints.load_plugin_spec_file()
             Endpoints.action_trigger_task_exists(plugin_spec_json, "triggers", name)
             return jsonify(
                 ActionTriggerDetailsSchema().dump(
@@ -559,8 +561,12 @@ class Endpoints:
                     jsonify({"error": "The specified format is not supported"}), 422
                 )
 
-            with open("/python/src/plugin.spec.yaml", "r") as p_spec:
-                plugin_spec = p_spec.read()
+            try:
+                with open(PLUGIN_SPEC_DOCKERFILE, "r") as p_spec:
+                    plugin_spec = p_spec.read()
+            except FileNotFoundError:
+                with open(PLUGIN_SPEC_PACK, "r") as p_spec:
+                    plugin_spec = p_spec.read()
 
             if format_ == "yaml":
                 return plugin_spec
@@ -744,7 +750,14 @@ class Endpoints:
         return actions_triggers_tasks.get(p_name)
 
     @staticmethod
-    def load_file_json_format(filename):
+    def load_plugin_spec_file() -> Dict[str, Any]:
+        try:
+            return Endpoints.load_file_json_format(PLUGIN_SPEC_DOCKERFILE)
+        except FileNotFoundError:
+            return Endpoints.load_file_json_format(PLUGIN_SPEC_PACK)
+
+    @staticmethod
+    def load_file_json_format(filename: str) -> Dict[str, Any]:
         with open(filename, "r") as plugin_spec:
             return yaml.safe_load(plugin_spec.read())
 
@@ -774,15 +787,14 @@ class Endpoints:
 
     def get_plugin_sdk_version(self):
         try:
-            version = pkg_resources.require("insightconnect-plugin-runtime")[0].version
+            version = importlib_metadata.version("insightconnect-plugin-runtime")
         except Exception:
             self.logger.warn("Unable to get SDK version")
             version = "0.0.0"
-
         return version
 
     def add_plugin_custom_config(
-        self, input_data: Dict[str, Any], org_id: str
+        self, input_data: Dict[str, Any], org_id: str, int_id: str, task_name: str
     ) -> Dict[str, Any]:
         """
         Using the retrieved configs pulled from komand-props, pass the configuration that matches the requesting
@@ -795,17 +807,95 @@ class Endpoints:
         - org 1 when in lookback mode will pull back 108 hours (task triggered with no state).
         - all orgs for default runs will poll back 12 hours.
         - all orgs in lookback mode will be 100 hours (task triggered with no state).
+
+        When Int ID is specified or task name. Then it can be used across all orgs (global config) or specific Org ID.
+        There's a hierarchy between task related config and Int ID related, where Int ID will always
+        replace task related config due to its higher priority.
+
+        Config example:
+        {
+            "org_1": {
+                "task_name_1": {
+                    "default": 12,
+                    "lookback": "100"
+                },
+                "int_1": {
+                    "default": 6,
+                    "lookback": "50"
+                },
+                {
+                    "default": 24,
+                    "lookback": "108"
+                }
+            },
+            "*": {
+                "default": 24,
+                "lookback": 200
+                "task_name_2": {
+                    "default": 2,
+                    "lookback": "10"
+                }
+            }
+        }
+
+        In this config example the following we be applied:
+        - org 1 to has a custom default time of 24 hours for their timings.
+        - org 1 when in lookback mode will pull back 108 hours (task triggered with no state).
+        - org 1 all integrations that uses task with 'task_name_1' will pull back 100 hours
+        and have a custom default time of 12 hours for their timings.
+        - org 1 specific integration 'int_1' will pull back 50 hours and have a custom
+        default time of 6 hours for their timings.
+        - all orgs for default runs will poll back 24 hours.
+        - all orgs in lookback mode will be 200 hours (task triggered with no state).
+        - all orgs that uses task with 'task_name_2' will pull back 2 hours
+        - all orgs that uses task with 'task_name_2' in lookback mode will be 10 hours
         """
-        additional_config = self.config_options.get(org_id) or self.config_options.get(
-            "*"
-        )
+
+        # Parse configuration based on organization or global. Use its copies, not to modify original dict.
+        organization_config, global_config = self.config_options.get(org_id, {}).copy(), self.config_options.get("*", {}).copy()
+
+        # Definition of additional config and its type variables.
+        additional_config, config_type = {}, ""
+
+        # Check if we have a global config.
+        # Also, use "config_type" variable just for logging purposes to have an indicator from where the config was pulled.
+        if global_config:
+            # Setup global config as starting point.
+            additional_config, config_type = global_config, "GLOBAL"
+
+            # If task configuration was found under global config, update it with task config to replace necessary values.
+            if task_config := global_config.get(task_name):
+                additional_config.update(task_config)
+                config_type = "GLOBAL_TASK"
+
+        # If organization config is present, then replace its values with the ones coming from global config.
+        if organization_config:
+            # Update additional config with values coming from organization config.
+            additional_config.update(organization_config)
+            config_type = "ORG"
+
+            # If task configuration was found under organization, replace its values with organization config.
+            if task_config := organization_config.get(task_name):
+                additional_config.update(task_config)
+                config_type = "ORG_TASK"
+
+            # If integration config was found under organization replace its values with organization
+            # and task config (higher priority).
+            if integration_config := organization_config.get(int_id):
+                additional_config.update(integration_config)
+                config_type = "ORG_INT"
+
         if additional_config:
             self.logger.info(
-                "Found config options; adding this to the request parameters..."
+                f"Found config options ({config_type}); adding this to the request parameters..."
             )
-            additional_config = (
-                additional_config.copy()
-            )  # copy to preserve the referenced value in self.config_options
+
+            # Sopy to preserve the referenced value in self.config_options.
+            # Also, remove unnecessary fields from that config (int_ids, or task_names) that occurs
+            # when updated global dictionary above. This thing is that we use global config as base
+            # and then updating some fields depending on the configuration.
+            additional_config = self._remove_unnecessary_fields_from_custom_config(additional_config)
+
             # As a safeguard we only pass the lookback config params if the plugin has no state
             # This means we still need to manually delete the state for plugins on a per org basis.
             # This also means first time customers for their 'initial' lookup would get the lookback value passed in.
@@ -821,7 +911,7 @@ class Endpoints:
 
         return input_data
 
-    def run_action_trigger_task(self, input_message, test=False, connection_test_type="test"):
+    def run_action_trigger_task(self, input_message, test=False, connection_test_type="test", mask_output=True):
         connection = input_message.get("body", {}).get("connection", {})
         status_code = 200
         output = None
@@ -850,7 +940,11 @@ class Endpoints:
                 status_code = 500
         finally:
             self.logger.debug("Request output: %s", output)
-            response = jsonify(OutputMasker.mask_output_data(connection, output))
+            # As this processing can take quite a while on large task ouputs we want to skip this
+            if mask_output:
+                response = jsonify(OutputMasker.mask_output_data(connection, output))
+            else:
+                response = jsonify(output)
             response.status_code = status_code
             return response
 
@@ -869,3 +963,46 @@ class Endpoints:
             return 501
         else:
             return 500
+
+    def _remove_unnecessary_fields_from_custom_config(self, additional_config: Dict[str, Any]) -> Dict[str, Any]:
+        """
+        Removes unnecessary fields from custom config such as other task names, and int_ids,
+        leaving only the fields that needs to be parsed.
+
+        :param additional_config: The custom config dictionary on which, unnecessary fields will be removed.
+        :type additional_config: Dict[str, Any]
+
+        :return: New custom config dictionary with unnecessary fields removed.
+        :rtype: Dict[str, Any]
+        """
+
+        # Copy 'additional_config' not to operate on it
+        config_copy = additional_config.copy()
+
+        # Remove other task names from config
+        for task_ in self.plugin.tasks.keys():
+            config_copy.pop(task_, None)
+
+        # Remove other int_ids from config
+        for key_ in additional_config.keys():
+            if self._check_if_uuid(key_):
+                config_copy.pop(key_, None)
+        return config_copy
+
+    @staticmethod
+    def _check_if_uuid(input_string: str) -> bool:
+        """
+        Validates whether the provided string matches UUID format specifications.
+
+        :param input_string: The string to validate against UUID format standards
+        :type input_string: str
+
+        :return: True if the string is a valid UUID, False otherwise
+        :rtype: bool
+        """
+
+        try:
+            UUID(input_string)
+            return True
+        except (TypeError, ValueError):
+            return False
