@@ -14,13 +14,26 @@ from botocore.exceptions import ClientError
 
 import insightconnect_plugin_runtime
 from insightconnect_plugin_runtime.action import Action
-from insightconnect_plugin_runtime.exceptions import PluginException
+from insightconnect_plugin_runtime.exceptions import (
+    PluginException,
+    ConnectionTestException,
+)
 from insightconnect_plugin_runtime.helper import clean
 
 REGION = "region"
 EXTERNAL_ID = "external_id"
 ROLE_ARN = "role_arn"
 ASSUME_ROLE_PARAMETERS = "assume_role_params"
+AWS_CLIENT_ERRORS = [
+    "AccessDeniedException",
+    "InternalServerException",
+    "ResourceNotFoundException",
+    "InternalFailure",
+    "InvalidClientTokenId",
+    "NotAuthorized",
+    "ServiceUnavailable",
+    "OptInRequired",
+]
 
 
 class PaginationHelper:
@@ -36,7 +49,7 @@ class PaginationHelper:
         limit_key: str = None,
         more_results: str = None,
         non_aggregate_keys: List[str] = None,
-        max_pages : int = None,  # if we want to limit how many pages of data we pull from AWS
+        max_pages: int = None,  # if we want to limit how many pages of data we pull from AWS
     ):
         self.input_token = input_token
         self.output_token = output_token
@@ -512,7 +525,9 @@ class AWSAction(Action):
         if self.pagination_helper:
             while self.pagination_helper.check_pagination(params, response):
                 if self.pagination_helper.check_total_results(params, response):
-                    self.logger.info("Reached max amount of pages per execution breaking...")
+                    self.logger.info(
+                        "Reached max amount of pages per execution breaking..."
+                    )
                     break
 
                 self.logger.info("Response was paginated. Performing another call.")
@@ -547,7 +562,28 @@ class AWSAction(Action):
         endpoint = client._endpoint.host  # pylint:disable=protected-access
         response = requests.get(endpoint)
 
-        assert response.ok  # noqa: B101
+        response_text = response.text
+        status_code = response.status_code
+
+        # AWS client errors are not always mapped predictably, and some service endpoints will return an unexpected
+        # result
+        if not (200 <= status_code <= 299):
+            if (
+                status_code in [401, 403, 404, 500, 503]
+                and "UnknownOperationException" not in response_text
+            ):
+                raise ConnectionTestException(
+                    cause=f"Error code {status_code} returned from AWS service",
+                    assistance="Please check the response for more information",
+                    data=response_text,
+                )
+            for error in AWS_CLIENT_ERRORS:
+                if error in response_text:
+                    raise ConnectionTestException(
+                        cause=f"Error code {status_code} returned from AWS service with {error} exception",
+                        assistance="Please check the response for more information",
+                        data=response_text,
+                    )
 
         if "properties" in self.output.schema:
             response = helper.format_output(self.output.schema, {})
@@ -563,7 +599,7 @@ class AWSAction(Action):
         auth_params: Dict[str, str],
     ):
         session_name = str(uuid.uuid1())
-        sts_client = boto3.client("sts", **auth_params)
+        sts_client = boto3.client("sts", **auth_params, region_name=assume_role_params.get(REGION))
         try:
             assumed_role_object = sts_client.assume_role(
                 **clean(
