@@ -10,7 +10,7 @@ import subprocess
 import time
 from datetime import datetime, timedelta
 from io import IOBase
-from typing import Any, Callable, Dict, List, Union, Tuple
+from typing import Any, Callable, Dict, List, Union, Tuple, Optional
 from urllib import request
 from hashlib import sha1
 from json import JSONDecodeError
@@ -21,6 +21,7 @@ from insightconnect_plugin_runtime.exceptions import (
     PluginException,
     HTTPStatusCodes,
     ResponseExceptionData,
+    APIException
 )
 
 CAMEL_CASE_REGEX = r"\b[a-z0-9]+([A-Z][a-z]+[0-9]*)*\b"
@@ -30,18 +31,48 @@ ENCODE_TYPE = "utf-8"
 
 DEFAULTS_HOURS_AGO = 24
 
+logger = logging.getLogger()
 
-def hash_sha1(log: Dict) -> str:
+
+def hash_sha1(log: Dict[str, Any], keys: Optional[List[str]] = None) -> str:
     """
     Iterate through a dictionary and hash each value.
+    Optionally only hash certain keys in the dictionary.
+
     :param log: Dictionary to be hashed.
-    :type Dict:
+    :param keys: Optional list of keys to hash on if provided
+
     :return: Hex digest of hash.
-    :rtype: str
     """
+
     hash_ = sha1()  # nosec B303
-    for key, value in log.items():
+
+    # Leaving no room for developer error and ensuring they know exactly where it went wrong
+    # if they provide a key not in list format
+    if keys is not None and not isinstance(keys, list):
+        raise TypeError(
+            f"The 'keys' parameter must be a list or None in the 'hash_sha1' function, not {type(keys).__name__}"
+        )
+
+    # Hash all key-value pairs if no keys provided
+    if keys is None:
+        items_to_hash = log.items()
+
+    # Otherwise, only include specified keys
+    else:
+        items_to_hash = []
+        for key in keys:
+            if key in log:
+                items_to_hash.append((key, log[key]))
+
+            # Alert if the key is not found in the log
+            else:
+                raise KeyError(f"Key '{key}' not found in the provided log.")
+
+    # Iterate through items to hash and hash
+    for key, value in items_to_hash:
         hash_.update(f"{key}{value}".encode(ENCODE_TYPE))
+
     return hash_.hexdigest()
 
 
@@ -65,7 +96,7 @@ def compare_and_dedupe_hashes(
         if hash_ not in previous_logs_hashes:
             new_logs_hashes.append(hash_)
             logs_to_return.append(log)
-    logging.info(
+    logger.info(
         f"Original number of logs:{len(new_logs)}. Number of logs after de-duplication:{len(logs_to_return)}"
     )
     return logs_to_return, new_logs_hashes
@@ -75,12 +106,13 @@ def make_request(
     _request: requests.Request,
     timeout: int = 60,
     verify: bool = True,
-    cert: Union[str, Tuple]=None,
+    cert: Union[str, Tuple] = None,
     stream: bool = False,
     allow_redirects: bool = True,
-    exception_custom_configs: Dict[int, Exception]={},
+    exception_custom_configs: Dict[int, Exception] = {},
     exception_data_location: str = None,
     allowed_status_codes: List[str] = [],
+    max_response_size: int = None,
 ) -> Tuple[requests.Response, Dict]:
     """
     Makes a HTTP request while checking for RequestErrors and JSONDecodeErrors
@@ -103,6 +135,8 @@ def make_request(
     :type str:
     :param allowed_status_codes: Status codes that will not raise an exception.
     :type List[str]:
+    :param max_response_size: Raise an error if the stream content is bigger than this specified size
+    :type int:
 
     :return: The request response and the response JSON.
     :rtype: Tuple[Response, Dict]
@@ -118,6 +152,19 @@ def make_request(
                 cert=cert,
                 stream=stream,
             )
+
+            # Before we close this session check and download all content, check if the returned content is too large.
+            # This may not be supported on all APIs as they need to support streaming and return the content-length
+            # header but adding this extra check will be beneficial for memory usage to those that do support it.
+            if stream and max_response_size:
+                resp_size = response.headers.get("content-length", "0")
+                if int(resp_size) > max_response_size:
+                    raise APIException(
+                        status_code=400,
+                        cause=f"API response is exceeding allowed limit of {max_response_size} bytes.",
+                        assistance="Please update the parameters to reduce the size of the data being returned.",
+                        data=f"Content length returned was {resp_size} and max allowed is {max_response_size}",
+                    )
     except requests.exceptions.Timeout as exception:
         raise PluginException(
             preset=PluginException.Preset.TIMEOUT, data=str(exception)
@@ -135,7 +182,12 @@ def make_request(
             raise PluginException(
                 preset=PluginException.Preset.UNKNOWN, data=str(exception)
             )
-    response_handler(response, exception_custom_configs, exception_data_location, allowed_status_codes)
+    response_handler(
+        response,
+        exception_custom_configs,
+        exception_data_location,
+        allowed_status_codes,
+    )
     return response
 
 
@@ -155,7 +207,11 @@ def extract_json(response: requests.Response) -> Dict:
         )
 
 
-def request_error_handling() -> Union[Any, None]:
+def request_error_handling(
+    custom_configs: Dict[int, Exception] = {},
+    exception_data_location: str = None,
+    allowed_status_codes: List[str] = [],
+) -> Union[Any, None]:
     """request_error_handling. This decorator allows a method that makes a request to complete with error handling.
     A plugin exception will be raised whenever an error is caught. Response.raise_for_status() must be called in the
     wrapped method to handle HTTPErrors.
@@ -182,7 +238,12 @@ def request_error_handling() -> Union[Any, None]:
                 )
             except requests.exceptions.RequestException as exception:
                 if isinstance(exception, requests.exceptions.HTTPError):
-                    response_handler(exception.response)
+                    response_handler(
+                        exception.response,
+                        data_location=exception_data_location,
+                        custom_configs=custom_configs,
+                        allowed_status_codes=allowed_status_codes,
+                    )
                 else:
                     raise PluginException(
                         preset=PluginException.Preset.UNKNOWN, data=str(exception)
@@ -195,13 +256,16 @@ def request_error_handling() -> Union[Any, None]:
 
 def response_handler(
     response: requests.Response,
-    custom_configs: Dict[int, Exception]={},
+    custom_configs: Dict[int, Exception] = {},
     data_location: str = None,
     allowed_status_codes: List[str] = [],
 ) -> None:
     """
     Check response status codes and return a generic PluginException preset if a HTTPError is raised.
-    Excetion cause, assistance, and data can be overwritten by supplied parameters.
+    Exception cause, assistance, and data can be overwritten by supplied parameters.
+
+    When we receive a common status code that we typically handle in the plugin, we raise an APIException
+    so that the status_code attribute can be easily accessed and raised further up the chain.
 
     :param response: Response object whose status should be checked.
     :type Response:
@@ -237,13 +301,13 @@ def response_handler(
         }
         status_code_preset = status_code_presets.get(status_code)
         exception = PluginException(preset=PluginException.Preset.UNKNOWN, data=data)
-        logging.info(f"Request to {response.url} failed. Status code: {status_code}")
+        logger.info(f"Request to {response.url} failed. Status code: {status_code}")
         if status_code in custom_configs.keys():
             exception = custom_configs.get(status_code)
             if hasattr(exception, "data") and data is not None:
                 exception.data = data
         elif status_code_preset:
-            exception = PluginException(preset=status_code_preset)
+            exception = APIException(preset=status_code_preset, status_code=status_code)
             exception.data = data
 
         raise exception
@@ -295,7 +359,7 @@ def extract_value(begin, key, end, s):
 
     Example: The following will use pull out the /bin/bash from the string s
     s = '\nShell: /bin/bash\n'
-    shell = get_value(r'\s', 'Shell', r':\s(.*)\s', s)
+    shell = get_value(r'\\s', 'Shell', r':\\s(.*)\\s', s)
 
     This function works well when you have a list of keys to iterate through where the pattern is the same.
     """
@@ -432,15 +496,19 @@ def convert_dict_to_snake_case(
 
     if isinstance(input_dict, list):
         return [
-            convert_dict_to_snake_case(element)
-            if isinstance(element, (dict, list))
-            else element
+            (
+                convert_dict_to_snake_case(element)
+                if isinstance(element, (dict, list))
+                else element
+            )
             for element in input_dict
         ]
     return {
-        convert_to_snake_case(key): convert_dict_to_snake_case(value)
-        if isinstance(value, (dict, list))
-        else value
+        convert_to_snake_case(key): (
+            convert_dict_to_snake_case(value)
+            if isinstance(value, (dict, list))
+            else value
+        )
         for key, value in input_dict.items()
     }
 
@@ -552,7 +620,7 @@ def rate_limiting(
                         error.cause
                         == PluginException.causes[PluginException.Preset.RATE_LIMIT]
                     ):
-                        logging.info(
+                        logger.info(
                             f"Rate limiting error occurred. Retrying in {delay:.1f} seconds ({attempts_counter}/{max_tries})"
                         )
                         retry = True
@@ -594,12 +662,12 @@ def check_hashes(src, checksum):
     if type(src) is str:
         hashes = get_hashes_string(src)
     else:
-        logging.error("CheckHashes: Argument must be a string")
+        logger.error("CheckHashes: Argument must be a string")
         raise Exception("CheckHashes")
     for alg in hashes:
         if hashes[alg] == checksum:
             return True
-    logging.info("CheckHashes: No checksum match")
+    logger.info("CheckHashes: No checksum match")
     return False
 
 
@@ -611,9 +679,9 @@ def check_cachefile(cache_file):
         cache_file = cache_dir + "/" + cache_file
     if os.path.isdir(cache_dir):
         if os.path.isfile(cache_file):
-            logging.info("CheckCacheFile: File %s exists", cache_file)
+            logger.info("CheckCacheFile: File %s exists", cache_file)
             return True
-        logging.info("CheckCacheFile: File %s did not exist", cache_file)
+        logger.info("CheckCacheFile: File %s did not exist", cache_file)
     return False
 
 
@@ -629,9 +697,9 @@ def open_file(file_path):
                 return f
             return None
         else:
-            logging.info("OpenFile: File %s is not a file or does not exist ", filename)
+            logger.info("OpenFile: File %s is not a file or does not exist ", filename)
     else:
-        logging.error(
+        logger.error(
             "OpenFile: Directory %s is not a directory or does not exist", dirname
         )
 
@@ -645,16 +713,16 @@ def open_cachefile(cache_file, append=False):
     if os.path.isdir(cache_dir):
         if os.path.isfile(cache_file):
             f = open(cache_file, "a+" if append else "r+")
-            logging.info("OpenCacheFile: %s exists, returning it", cache_file)
+            logger.info("OpenCacheFile: %s exists, returning it", cache_file)
         else:
             if not os.path.isdir(os.path.dirname(cache_file)):
                 os.makedirs(os.path.dirname(cache_file))
             f = open(cache_file, "w+")  # Open once to create the cache file
             f.close()
-            logging.info("OpenCacheFile: %s created", cache_file)
+            logger.info("OpenCacheFile: %s created", cache_file)
             f = open(cache_file, "a+" if append else "r+")
         return f
-    logging.error("OpenCacheFile: %s directory or does not exist", cache_dir)
+    logger.error("OpenCacheFile: %s directory or does not exist", cache_dir)
 
 
 def remove_cachefile(cache_file):
@@ -667,7 +735,7 @@ def remove_cachefile(cache_file):
         if os.path.isfile(cache_file):
             os.remove(cache_file)
             return True
-        logging.info("RemoveCacheFile: Cache file %s did not exist", cache_file)
+        logger.info("RemoveCacheFile: Cache file %s did not exist", cache_file)
     return False
 
 
@@ -686,9 +754,9 @@ def lock_cache(lock_file):
             os.makedirs(os.path.dirname(lock_file))
         f = open(lock_file, "w")
         f.close()
-        logging.info("Cache lock %s created", lock_file)
+        logger.info("Cache lock %s created", lock_file)
         return True
-    logging.info("Cache lock %s failed, lock not created", lock_file)
+    logger.info("Cache lock %s failed, lock not created", lock_file)
     return False
 
 
@@ -707,7 +775,7 @@ def unlock_cache(lock_file, wait_time):
             time.sleep(wait_time)
             os.remove(lock_file)
             return True
-        logging.info("Cache unlock %s failed, lock not released", lock_file)
+        logger.info("Cache unlock %s failed, lock not released", lock_file)
     return False
 
 
@@ -737,11 +805,11 @@ def open_url(url, timeout=None, verify=True, **kwargs):
             urlobj = request.urlopen(req, timeout=timeout, context=ctx)
         return urlobj
     except request.HTTPError as e:
-        logging.error("HTTPError: %s for %s", str(e.code), url)
+        logger.error("HTTPError: %s for %s", str(e.code), url)
         if e.code == 304:
             return None
     except request.URLError as e:
-        logging.error("URLError: %s for %s", str(e.reason), url)
+        logger.error("URLError: %s for %s", str(e.reason), url)
     raise Exception("GetURL Failed")
 
 
@@ -769,17 +837,17 @@ def check_url(url):
             return True
 
     except requests.exceptions.HTTPError:
-        logging.error(
+        logger.error(
             "Requests: HTTPError: status code %s for %s",
             str(resp.status_code) if resp else None,
             url,
         )
     except requests.exceptions.Timeout:
-        logging.error("Requests: Timeout for %s", url)
+        logger.error("Requests: Timeout for %s", url)
     except requests.exceptions.TooManyRedirects:
-        logging.error("Requests: TooManyRedirects for %s", url)
+        logger.error("Requests: TooManyRedirects for %s", url)
     except requests.ConnectionError:
-        logging.error("Requests: ConnectionError for %s", url)
+        logger.error("Requests: ConnectionError for %s", url)
     return False
 
 
@@ -799,7 +867,7 @@ def exec_command(command):
         rcode = p.poll()
         return {"stdout": stdout, "stderr": stderr, "rcode": rcode}
     except OSError as e:
-        logging.error(
+        logger.error(
             "SubprocessError: %s %s: %s", str(e.filename), str(e.strerror), str(e.errno)
         )
     raise Exception("ExecCommand")
@@ -825,7 +893,7 @@ def encode_file(file_path):
             return efile
         return None
     except (IOError, OSError) as e:
-        logging.error("EncodeFile: Failed to open file: %s", e.strerror)
+        logger.error("EncodeFile: Failed to open file: %s", e.strerror)
         raise Exception("EncodeFile")
     finally:
         if isinstance(f, IOBase):
@@ -848,17 +916,17 @@ def check_url_modified(url):
         if resp.status_code == 200:
             return True
     except requests.exceptions.HTTPError:
-        logging.error(
+        logger.error(
             "Requests: HTTPError: status code %s for %s",
             str(resp.status_code) if resp else None,
             url,
         )
     except requests.exceptions.Timeout:
-        logging.error("Requests: Timeout for %s", url)
+        logger.error("Requests: Timeout for %s", url)
     except requests.exceptions.TooManyRedirects:
-        logging.error("Requests: TooManyRedirects for %s", url)
+        logger.error("Requests: TooManyRedirects for %s", url)
     except requests.ConnectionError:
-        logging.error("Requests: ConnectionError for %s", url)
+        logger.error("Requests: ConnectionError for %s", url)
     return False
 
 
@@ -884,7 +952,7 @@ def get_url_path_filename(url):
             if name[n].endswith("."):
                 return name
     except IndexError:
-        logging.error("Range: IndexError: URL basename is short: %s of %s", name, url)
+        logger.error("Range: IndexError: URL basename is short: %s of %s", name, url)
         return None
     return None
 
@@ -904,16 +972,16 @@ def get_url_filename(url):
             return name
         return None
     except requests.exceptions.MissingSchema:
-        logging.error("Requests: MissingSchema: Requires ftp|http(s):// for %s", url)
+        logger.error("Requests: MissingSchema: Requires ftp|http(s):// for %s", url)
     except requests.exceptions.HTTPError:
-        logging.error(
+        logger.error(
             "Requests: HTTPError: status code %s for %s",
             str(resp.status_code) if resp else None,
             url,
         )
     except requests.exceptions.Timeout:
-        logging.error("Requests: Timeout for %s", url)
+        logger.error("Requests: Timeout for %s", url)
     except requests.exceptions.TooManyRedirects:
-        logging.error("Requests: TooManyRedirects for %s", url)
+        logger.error("Requests: TooManyRedirects for %s", url)
     except requests.ConnectionError:
-        logging.error("Requests: ConnectionError for %s", url)
+        logger.error("Requests: ConnectionError for %s", url)
