@@ -402,6 +402,9 @@ class Plugin(object):
         status_code = None
         error_object = None
 
+        # Properties specific to tasks tests
+        task_test_log = None
+
         try:
             # Attempt to grab message type first
             message_type = input_message.get("type")
@@ -444,7 +447,7 @@ class Plugin(object):
                 out_type = "task_event"
                 if is_test:
                     # state will not be returned by task's test method
-                    output = self.start_step(
+                    output, task_test_log = self.start_step(
                         input_message["body"],
                         "task",
                         struct_logger,
@@ -471,7 +474,7 @@ class Plugin(object):
                     )
             elif message_type == "connection_test":
                 out_type = "connection_test"
-                output = self.start_step(
+                output, task_test_log = self.start_step(
                     input_message["body"],
                     "connection_test",
                     struct_logger,
@@ -481,15 +484,28 @@ class Plugin(object):
                     is_connection_test=True,
                     connection_test_type=connection_test_type
                 )
-        except (ClientException, ServerException, PluginException, Exception) as e:
+        except (
+            ClientException,
+            ServerException,
+            PluginException,
+            ConnectionTestException,
+            Exception,
+        ) as error_message:
             success = False
-            caught_exception = e
-            struct_logger.exception(e)
+            caught_exception = error_message
+            struct_logger.exception(error_message)
+
+            # now returning the value of data instead of the log buffer for connection task tests - SOAR-16566
+            if isinstance(error_message, ConnectionTestException) and connection_test_type == "test_task":
+                task_test_log = f"{error_message.data}" if error_message.data else "Connection test failed"
         finally:
+            # if we are running a task connection test we want to return the pre-defined message
+            # rather than a stack trace of logs - SOAR-16566
+            log_message = task_test_log if task_test_log else log_stream.getvalue()
             output = self.envelope(
                 out_type,
                 input_message,
-                log_stream.getvalue(),
+                log_message,
                 success,
                 output,
                 str(caught_exception),
@@ -541,18 +557,25 @@ class Plugin(object):
                     plugin_version=connection.meta.version,
                 )
             )
+
+            # As the message type for both api/v1/connection/test and api/v1/connection/test_task calls are
+            # message_type == "connection_test", this means that both the task and normal connection test code
+            # hits this path and needs to have the same return structure, as a result we return None when
+            # connection.test is called, as there is no message returned along side it,
+            # and the code above then sees this and reads the "message" from the io buffer as normal
             if hasattr(connection, "test_task") and connection_test_type == "test_task":
                 func = connection.test_task
+                output, log = func()
+                return output, log
             elif hasattr(connection, "test"):
                 func = connection.test
+                output = func()
+                return output, None
             else:
                 raise NotImplementedError(
                     "The server successfully processed the request and is not "
                     "returning any content (no connection test function)"
                 )
-            output = func()
-
-            return output
 
         else:
             action_name = message_body[step_key]
@@ -626,8 +649,10 @@ class Plugin(object):
             if is_test:
                 # Check if connection test func available. If so - use it (preferred). Else fallback to action/trigger test
                 if hasattr(step.connection, "test"):
-                    if hasattr(step.connection, "test") and connection_test_type == "test_task":
+                    if hasattr(step.connection, "test_task") and connection_test_type == "test_task":
                         func = step.connection.test_task
+                        output, log = func()
+                        return output, log
                     else:
                         func = step.connection.test
                 else:
