@@ -4,6 +4,7 @@ from insightconnect_plugin_runtime.exceptions import (
     HTTPStatusCodes,
     ResponseExceptionData,
     PluginException,
+    APIException
 )
 import requests
 import os
@@ -378,16 +379,39 @@ class TestRequestsHelpers(TestCase):
             json={"sample": "value"},
             headers={"Content-Type": "application/json"},
         )
-        response = helper.make_request(_request=request)
+        response = helper.make_request(_request=request, stream=True, max_response_size=100)
         expected = {
             "json": {"example": "sample"},
             "status_code": 200,
             "content": b"example",
             "url": "https://example.com/success",
+            "content-length": "1",
         }
         self.assertEqual(response.content, expected.get("content"))
         self.assertEqual(response.status_code, expected.get("status_code"))
         self.assertEqual(response.url, expected.get("url"))
+        self.assertEqual(response.headers.get("content-length"), expected.get("content-length"))
+
+    @patch("requests.Session.send")
+    def test_make_request_enforces_max_response_size(self, mocked_request):
+        returned_max_size, test_max_size = "5000", 1527
+
+        response = requests.Response()
+        response.headers = {"content-length": returned_max_size, "content-type": "application/json"}
+        mocked_request.return_value = response
+
+        test_request = requests.Request(method="GET", url="https://event_source.com/api/v1/logs")
+
+        with self.assertRaises(APIException) as api_err:
+            helper.make_request(_request=test_request, stream=True, max_response_size=test_max_size)
+
+        self.assertEqual(400, api_err.exception.status_code)
+        exp_err_cause = f"API response is exceeding allowed limit of {test_max_size} bytes."
+        exp_err_data = f"Content length returned was {returned_max_size} and max allowed is {test_max_size}"
+        self.assertEqual(400, api_err.exception.status_code)
+        self.assertEqual(exp_err_cause, api_err.exception.cause)
+        self.assertEqual(exp_err_data, api_err.exception.data)
+
 
     @parameterized.expand(
         [
@@ -510,20 +534,46 @@ class TestRequestsHelpers(TestCase):
         )
 
 
-def test_hash_sha1():
-    log = {"example": "value", "sample": "value"}
-    assert "2e1ccc1a95e9b2044f13546c25fe380bbd039293" == helper.hash_sha1(log)
+class TestHashing(TestCase):
+    def setUp(self) -> None:
+        self.log = {"example": "value", "sample": "value"}
 
+    def test_hash_sha1_no_keys(self):
+        # Test hash with no keys provided
+        expected_hash = "2e1ccc1a95e9b2044f13546c25fe380bbd039293"
+        self.assertEqual(helper.hash_sha1(self.log), expected_hash)
 
-def test_compare_and_dedupe_hashes():
-    hashes = ["2e1ccc1a95e9b2044f13546c25fe380bbd039293"]
-    logs = [
-        {
-            "example": "value",
-            "sample": "value",
-        },
-        {"specimen": "new_value"},
-    ]
-    assert [{"specimen": "new_value"}], [
-        "ad6ae80c0356e02b1561cb58408ee678eb1070bb"
-    ] == helper.compare_and_dedupe_hashes(hashes, logs)
+    def test_hash_sha1_keys(self):
+        # Test hash with valid key provided
+        expected_hash = "61c908e52d66a763ceed0798b8e5f4b7f0328a21"
+        self.assertEqual(helper.hash_sha1(self.log, keys=["example"]), expected_hash)
+
+    def test_hash_sha1_keys_wrong_type(self):
+        # Test hash with wrong type for keys
+        with self.assertRaises(TypeError) as context:
+            helper.hash_sha1(self.log, keys="test")
+
+        self.assertEqual(
+            str(context.exception),
+            "The 'keys' parameter must be a list or None in the 'hash_sha1' function, not str"
+        )
+
+    def test_hash_sha1_keys_not_found(self):
+        # Test hash with key not found
+        with self.assertRaises(KeyError) as context:
+            helper.hash_sha1(self.log, keys=["example", "test"])
+
+        self.assertEqual(str(context.exception), "\"Key 'test' not found in the provided log.\"")
+
+    def test_compare_and_dedupe_hashes(self):
+        hashes = ["2e1ccc1a95e9b2044f13546c25fe380bbd039293"]
+        logs = [
+            {
+                "example": "value",
+                "sample": "value",
+            },
+            {"specimen": "new_value"},
+        ]
+        assert [{"specimen": "new_value"}], [
+            "ad6ae80c0356e02b1561cb58408ee678eb1070bb"
+        ] == helper.compare_and_dedupe_hashes(hashes, logs)
